# Generated from C:/Users/oscar/Downloads/Development/TL\CircuitSAT.g4 by ANTLR 4.10.1
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .CircuitSATParser import CircuitSATParser
else:
    from CircuitSATParser import CircuitSATParser

# This class defines a complete listener for a parse tree produced by CircuitSATParser.
class CircuitSATListener(ParseTreeListener):

    # Enter a parse tree produced by CircuitSATParser#circuit.
    def enterCircuit(self, ctx:CircuitSATParser.CircuitContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#circuit.
    def exitCircuit(self, ctx:CircuitSATParser.CircuitContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#block.
    def enterBlock(self, ctx:CircuitSATParser.BlockContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#block.
    def exitBlock(self, ctx:CircuitSATParser.BlockContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#statement.
    def enterStatement(self, ctx:CircuitSATParser.StatementContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#statement.
    def exitStatement(self, ctx:CircuitSATParser.StatementContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#assignment.
    def enterAssignment(self, ctx:CircuitSATParser.AssignmentContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#assignment.
    def exitAssignment(self, ctx:CircuitSATParser.AssignmentContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#identifierFunctionCall.
    def enterIdentifierFunctionCall(self, ctx:CircuitSATParser.IdentifierFunctionCallContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#identifierFunctionCall.
    def exitIdentifierFunctionCall(self, ctx:CircuitSATParser.IdentifierFunctionCallContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#printlnFunctionCall.
    def enterPrintlnFunctionCall(self, ctx:CircuitSATParser.PrintlnFunctionCallContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#printlnFunctionCall.
    def exitPrintlnFunctionCall(self, ctx:CircuitSATParser.PrintlnFunctionCallContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#printFunctionCall.
    def enterPrintFunctionCall(self, ctx:CircuitSATParser.PrintFunctionCallContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#printFunctionCall.
    def exitPrintFunctionCall(self, ctx:CircuitSATParser.PrintFunctionCallContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#assertFunctionCall.
    def enterAssertFunctionCall(self, ctx:CircuitSATParser.AssertFunctionCallContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#assertFunctionCall.
    def exitAssertFunctionCall(self, ctx:CircuitSATParser.AssertFunctionCallContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#sizeFunctionCall.
    def enterSizeFunctionCall(self, ctx:CircuitSATParser.SizeFunctionCallContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#sizeFunctionCall.
    def exitSizeFunctionCall(self, ctx:CircuitSATParser.SizeFunctionCallContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#ifStatement.
    def enterIfStatement(self, ctx:CircuitSATParser.IfStatementContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#ifStatement.
    def exitIfStatement(self, ctx:CircuitSATParser.IfStatementContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#ifStat.
    def enterIfStat(self, ctx:CircuitSATParser.IfStatContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#ifStat.
    def exitIfStat(self, ctx:CircuitSATParser.IfStatContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#elseIfStat.
    def enterElseIfStat(self, ctx:CircuitSATParser.ElseIfStatContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#elseIfStat.
    def exitElseIfStat(self, ctx:CircuitSATParser.ElseIfStatContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#elseStat.
    def enterElseStat(self, ctx:CircuitSATParser.ElseStatContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#elseStat.
    def exitElseStat(self, ctx:CircuitSATParser.ElseStatContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#functionDecl.
    def enterFunctionDecl(self, ctx:CircuitSATParser.FunctionDeclContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#functionDecl.
    def exitFunctionDecl(self, ctx:CircuitSATParser.FunctionDeclContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#forStatement.
    def enterForStatement(self, ctx:CircuitSATParser.ForStatementContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#forStatement.
    def exitForStatement(self, ctx:CircuitSATParser.ForStatementContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#whileStatement.
    def enterWhileStatement(self, ctx:CircuitSATParser.WhileStatementContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#whileStatement.
    def exitWhileStatement(self, ctx:CircuitSATParser.WhileStatementContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#assemblerStatement.
    def enterAssemblerStatement(self, ctx:CircuitSATParser.AssemblerStatementContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#assemblerStatement.
    def exitAssemblerStatement(self, ctx:CircuitSATParser.AssemblerStatementContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#idList.
    def enterIdList(self, ctx:CircuitSATParser.IdListContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#idList.
    def exitIdList(self, ctx:CircuitSATParser.IdListContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#exprList.
    def enterExprList(self, ctx:CircuitSATParser.ExprListContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#exprList.
    def exitExprList(self, ctx:CircuitSATParser.ExprListContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#boolExpression.
    def enterBoolExpression(self, ctx:CircuitSATParser.BoolExpressionContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#boolExpression.
    def exitBoolExpression(self, ctx:CircuitSATParser.BoolExpressionContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#numberExpression.
    def enterNumberExpression(self, ctx:CircuitSATParser.NumberExpressionContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#numberExpression.
    def exitNumberExpression(self, ctx:CircuitSATParser.NumberExpressionContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#identifierExpression.
    def enterIdentifierExpression(self, ctx:CircuitSATParser.IdentifierExpressionContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#identifierExpression.
    def exitIdentifierExpression(self, ctx:CircuitSATParser.IdentifierExpressionContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#notExpression.
    def enterNotExpression(self, ctx:CircuitSATParser.NotExpressionContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#notExpression.
    def exitNotExpression(self, ctx:CircuitSATParser.NotExpressionContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#orExpression.
    def enterOrExpression(self, ctx:CircuitSATParser.OrExpressionContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#orExpression.
    def exitOrExpression(self, ctx:CircuitSATParser.OrExpressionContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#unaryMinusExpression.
    def enterUnaryMinusExpression(self, ctx:CircuitSATParser.UnaryMinusExpressionContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#unaryMinusExpression.
    def exitUnaryMinusExpression(self, ctx:CircuitSATParser.UnaryMinusExpressionContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#powerExpression.
    def enterPowerExpression(self, ctx:CircuitSATParser.PowerExpressionContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#powerExpression.
    def exitPowerExpression(self, ctx:CircuitSATParser.PowerExpressionContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#eqExpression.
    def enterEqExpression(self, ctx:CircuitSATParser.EqExpressionContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#eqExpression.
    def exitEqExpression(self, ctx:CircuitSATParser.EqExpressionContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#andExpression.
    def enterAndExpression(self, ctx:CircuitSATParser.AndExpressionContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#andExpression.
    def exitAndExpression(self, ctx:CircuitSATParser.AndExpressionContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#inExpression.
    def enterInExpression(self, ctx:CircuitSATParser.InExpressionContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#inExpression.
    def exitInExpression(self, ctx:CircuitSATParser.InExpressionContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#stringExpression.
    def enterStringExpression(self, ctx:CircuitSATParser.StringExpressionContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#stringExpression.
    def exitStringExpression(self, ctx:CircuitSATParser.StringExpressionContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#expressionExpression.
    def enterExpressionExpression(self, ctx:CircuitSATParser.ExpressionExpressionContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#expressionExpression.
    def exitExpressionExpression(self, ctx:CircuitSATParser.ExpressionExpressionContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#addExpression.
    def enterAddExpression(self, ctx:CircuitSATParser.AddExpressionContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#addExpression.
    def exitAddExpression(self, ctx:CircuitSATParser.AddExpressionContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#compExpression.
    def enterCompExpression(self, ctx:CircuitSATParser.CompExpressionContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#compExpression.
    def exitCompExpression(self, ctx:CircuitSATParser.CompExpressionContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#nullExpression.
    def enterNullExpression(self, ctx:CircuitSATParser.NullExpressionContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#nullExpression.
    def exitNullExpression(self, ctx:CircuitSATParser.NullExpressionContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#functionCallExpression.
    def enterFunctionCallExpression(self, ctx:CircuitSATParser.FunctionCallExpressionContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#functionCallExpression.
    def exitFunctionCallExpression(self, ctx:CircuitSATParser.FunctionCallExpressionContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#multExpression.
    def enterMultExpression(self, ctx:CircuitSATParser.MultExpressionContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#multExpression.
    def exitMultExpression(self, ctx:CircuitSATParser.MultExpressionContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#listExpression.
    def enterListExpression(self, ctx:CircuitSATParser.ListExpressionContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#listExpression.
    def exitListExpression(self, ctx:CircuitSATParser.ListExpressionContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#ternaryExpression.
    def enterTernaryExpression(self, ctx:CircuitSATParser.TernaryExpressionContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#ternaryExpression.
    def exitTernaryExpression(self, ctx:CircuitSATParser.TernaryExpressionContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#inputExpression.
    def enterInputExpression(self, ctx:CircuitSATParser.InputExpressionContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#inputExpression.
    def exitInputExpression(self, ctx:CircuitSATParser.InputExpressionContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#list.
    def enterList(self, ctx:CircuitSATParser.ListContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#list.
    def exitList(self, ctx:CircuitSATParser.ListContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#indexes.
    def enterIndexes(self, ctx:CircuitSATParser.IndexesContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#indexes.
    def exitIndexes(self, ctx:CircuitSATParser.IndexesContext):
        pass


    # Enter a parse tree produced by CircuitSATParser#opcode.
    def enterOpcode(self, ctx:CircuitSATParser.OpcodeContext):
        pass

    # Exit a parse tree produced by CircuitSATParser#opcode.
    def exitOpcode(self, ctx:CircuitSATParser.OpcodeContext):
        pass



del CircuitSATParser


from circuitsat.CircuitSATParser import CircuitSATParser
from circuitsat.CircuitSATVisitor import CircuitSATVisitor


class CicuitSAT(CircuitSATVisitor):
    def __init__(self, file_name):
        self.file_name = file_name
        self.cnf_file = open(self.file_name, 'w+')
        self.current_literal = 0
        self.current_clause = 0
        self.stack = []
        self.register = 0
        self.store = {}

    def add_clauses(self, clauses):
        for cls in clauses:
            self.current_clause += 1
            print(' '.join(list(map(str, cls))) + ' 0', file=self.cnf_file)

    def add_comment(self, comment):
        self.current_clause += 1
        print('c {}'.format(comment), file=self.cnf_file)

    def new(self):
        self.current_literal += 1
        self.register = self.current_literal
        return self.register

    def push(self):
        self.stack.append(self.register)

    def pop(self):
        lit = self.stack[-1]
        del self.stack[-1]
        self.register = lit
        return self.register

    def top(self):
        return self.register

    def apply_true(self):
        x = self.pop()
        self.add_clauses([[x]])
        self.register = x
        self.push()

    def apply_false(self):
        x = self.pop()
        self.add_clauses([[-x]])
        self.register = x
        self.push()

    def apply_not(self):
        x = self.pop()
        y = self.pop()
        self.add_clauses([[x, y],
                          [-x, -y]])
        self.register = y

    def apply_copy(self):
        x = self.pop()
        y = self.pop()
        self.add_clauses([[x, -y],
                          [-x, y]])
        self.register = y

    def apply_and(self):
        x = self.pop()
        y = self.pop()
        z = self.pop()
        self.add_clauses([[x, y, -z],
                          [x, -y, -z],
                          [-x, y, -z],
                          [-x, -y, z]])
        self.register = z

    def apply_nand(self):
        x = self.pop()
        y = self.pop()
        z = self.pop()
        self.add_clauses([[x, y, z],
                          [x, -y, z],
                          [-x, y, z],
                          [-x, -y, -z]])
        self.register = z

    def apply_or(self):
        x = self.pop()
        y = self.pop()
        z = self.pop()
        self.add_clauses([[x, y, -z],
                          [x, -y, z],
                          [-x, y, z],
                          [-x, -y, z]])
        self.register = z

    def apply_nor(self):
        x = self.pop()
        y = self.pop()
        z = self.pop()
        self.add_clauses([[x, y, z],
                          [x, -y, -z],
                          [-x, y, -z],
                          [-x, -y, -z]])
        self.register = z

    def apply_xor(self):
        x = self.pop()
        y = self.pop()
        z = self.pop()
        self.add_clauses([[x, y, -z],
                          [x, -y, z],
                          [-x, y, z],
                          [-x, -y, -z]])
        self.register = z

    def apply_equ(self):
        x = self.pop()
        y = self.pop()
        z = self.pop()
        self.add_clauses([[x, y, z],
                          [x, -y, -z],
                          [-x, y, -z],
                          [-x, -y, z]])
        self.register = z

    def apply_set(self, name):
        self.store[name] = self.register

    def apply_get(self, name):
        self.register = self.store[name]

    def apply_cur(self, name):
        if name in self.store:
            self.current_literal = self.store[name]
        else:
            self.current_literal = int(name) - 1

    def visitOpcode(self, ctx: CircuitSATParser.OpcodeContext):
        opcode = ctx.getText().lower()
        if opcode == 'new':
            self.new()
        elif opcode == 'push':
            self.push()
        elif opcode == 'pop':
            self.pop()
        elif opcode.startswith('set'):
            self.apply_set(opcode[len('set'):].strip())
        elif opcode.startswith('get'):
            self.apply_get(opcode[len('get'):].strip())
        elif opcode.startswith('cur'):
            self.apply_cur(opcode[len('cur'):].strip())
        elif opcode == 'true':
            self.apply_true()
        elif opcode == 'false':
            self.apply_false()
        elif opcode == 'copy':
            self.apply_copy()
        elif opcode == 'not':
            self.apply_not()
        elif opcode == 'and':
            self.apply_and()
        elif opcode == 'or':
            self.apply_or()
        elif opcode == 'xor':
            self.apply_xor()
        elif opcode == 'equ':
            self.apply_equ()
        elif opcode == 'nor':
            self.apply_nor()
        elif opcode == 'nand':
            self.apply_nand()
        return super().visitOpcode(ctx)

    def visitParse(self, ctx: CircuitSATParser.ParseContext):
        ret = super().visitParse(ctx)
        self.cnf_file.close()
        cnf_file = open(self.file_name, 'r+')
        header = 'c CircuitSAT Assembler\n' + 'p cnf {} {}'.format(self.current_literal, self.current_clause)
        content = cnf_file.read()
        cnf_file.seek(0, 0)
        cnf_file.write(header.rstrip('\r\n') + '\n' + content)
        cnf_file.close()
        return ret

    def visitBlock(self, ctx: CircuitSATParser.BlockContext):
        return super().visitBlock(ctx)

    def visitStatement(self, ctx: CircuitSATParser.StatementContext):
        return super().visitStatement(ctx)

    def visitAssignment(self, ctx: CircuitSATParser.AssignmentContext):
        return super().visitAssignment(ctx)

    def visitIdentifierFunctionCall(self, ctx: CircuitSATParser.IdentifierFunctionCallContext):
        return super().visitIdentifierFunctionCall(ctx)

    def visitPrintlnFunctionCall(self, ctx: CircuitSATParser.PrintlnFunctionCallContext):
        return super().visitPrintlnFunctionCall(ctx)

    def visitPrintFunctionCall(self, ctx: CircuitSATParser.PrintFunctionCallContext):
        return super().visitPrintFunctionCall(ctx)

    def visitAssertFunctionCall(self, ctx: CircuitSATParser.AssertFunctionCallContext):
        return super().visitAssertFunctionCall(ctx)

    def visitSizeFunctionCall(self, ctx: CircuitSATParser.SizeFunctionCallContext):
        return super().visitSizeFunctionCall(ctx)

    def visitIfStatement(self, ctx: CircuitSATParser.IfStatementContext):
        return super().visitIfStatement(ctx)

    def visitIfStat(self, ctx: CircuitSATParser.IfStatContext):
        return super().visitIfStat(ctx)

    def visitElseIfStat(self, ctx: CircuitSATParser.ElseIfStatContext):
        return super().visitElseIfStat(ctx)

    def visitElseStat(self, ctx: CircuitSATParser.ElseStatContext):
        return super().visitElseStat(ctx)

    def visitFunctionDecl(self, ctx: CircuitSATParser.FunctionDeclContext):
        return super().visitFunctionDecl(ctx)

    def visitForStatement(self, ctx: CircuitSATParser.ForStatementContext):
        return super().visitForStatement(ctx)

    def visitWhileStatement(self, ctx: CircuitSATParser.WhileStatementContext):
        return super().visitWhileStatement(ctx)

    def visitIdList(self, ctx: CircuitSATParser.IdListContext):
        return super().visitIdList(ctx)

    def visitExprList(self, ctx: CircuitSATParser.ExprListContext):
        return super().visitExprList(ctx)

    def visitBoolExpression(self, ctx: CircuitSATParser.BoolExpressionContext):
        return super().visitBoolExpression(ctx)

    def visitNumberExpression(self, ctx: CircuitSATParser.NumberExpressionContext):
        return super().visitNumberExpression(ctx)

    def visitIdentifierExpression(self, ctx: CircuitSATParser.IdentifierExpressionContext):
        return super().visitIdentifierExpression(ctx)

    def visitNotExpression(self, ctx: CircuitSATParser.NotExpressionContext):
        return super().visitNotExpression(ctx)

    def visitOrExpression(self, ctx: CircuitSATParser.OrExpressionContext):
        return super().visitOrExpression(ctx)

    def visitUnaryMinusExpression(self, ctx: CircuitSATParser.UnaryMinusExpressionContext):
        return super().visitUnaryMinusExpression(ctx)

    def visitPowerExpression(self, ctx: CircuitSATParser.PowerExpressionContext):
        return super().visitPowerExpression(ctx)

    def visitEqExpression(self, ctx: CircuitSATParser.EqExpressionContext):
        return super().visitEqExpression(ctx)

    def visitAndExpression(self, ctx: CircuitSATParser.AndExpressionContext):
        return super().visitAndExpression(ctx)

    def visitInExpression(self, ctx: CircuitSATParser.InExpressionContext):
        return super().visitInExpression(ctx)

    def visitStringExpression(self, ctx: CircuitSATParser.StringExpressionContext):
        return super().visitStringExpression(ctx)

    def visitExpressionExpression(self, ctx: CircuitSATParser.ExpressionExpressionContext):
        return super().visitExpressionExpression(ctx)

    def visitAddExpression(self, ctx: CircuitSATParser.AddExpressionContext):
        return super().visitAddExpression(ctx)

    def visitCompExpression(self, ctx: CircuitSATParser.CompExpressionContext):
        return super().visitCompExpression(ctx)

    def visitNullExpression(self, ctx: CircuitSATParser.NullExpressionContext):
        return super().visitNullExpression(ctx)

    def visitFunctionCallExpression(self, ctx: CircuitSATParser.FunctionCallExpressionContext):
        return super().visitFunctionCallExpression(ctx)

    def visitMultExpression(self, ctx: CircuitSATParser.MultExpressionContext):
        return super().visitMultExpression(ctx)

    def visitListExpression(self, ctx: CircuitSATParser.ListExpressionContext):
        return super().visitListExpression(ctx)

    def visitTernaryExpression(self, ctx: CircuitSATParser.TernaryExpressionContext):
        return super().visitTernaryExpression(ctx)

    def visitInputExpression(self, ctx: CircuitSATParser.InputExpressionContext):
        return super().visitInputExpression(ctx)

    def visitList(self, ctx: CircuitSATParser.ListContext):
        return super().visitList(ctx)

    def visitIndexes(self, ctx: CircuitSATParser.IndexesContext):
        return super().visitIndexes(ctx)




import copy
from circuitsat.CircuitSATLexer import CircuitSATLexer
from circuitsat.CircuitSATParser import CircuitSATParser
from circuitsat.CircuitSATVisitor import CircuitSATVisitor


class ReturnValue(Exception):
    def __init__(self, value):
        self.value = value


class Function:
    def __init__(self, idx, args, block):
        self.idx = idx
        self.args = args
        self.block = block

    def invoke(self, master, args):
        if master.recursive:
            slave = CicuitSAT(master.file_name)
            slave.store = copy.copy(master.store)
            slave.cnf_file = master.cnf_file
            slave.current_literal = master.current_literal
            slave.current_clause = master.current_clause
        else:
            slave = master
        for arg, value in zip(self.args, args):
            slave.store[arg.getText()] = master.visit(value)
        try:
            ret = slave.visit(self.block)
            master.current_literal = slave.current_literal
            master.current_clause = slave.current_clause
            return ret
        except ReturnValue as ret:
            master.current_literal = slave.current_literal
            master.current_clause = slave.current_clause
            return ret.value


class CicuitSAT(CircuitSATVisitor):
    def __init__(self, file_name):
        self.file_name = file_name
        self.cnf_file = open(self.file_name, 'w+')
        self.current_literal = 0
        self.current_clause = 0
        self.store = {}
        self.recursive = True

    def visitCircuit(self, ctx: CircuitSATParser.CircuitContext):
        ret = super().visitCircuit(ctx)
        self.cnf_file.close()
        cnf_file = open(self.file_name, 'r+')
        header = 'c CircuitSAT Language\n' + 'p cnf {} {}'.format(self.current_literal, self.current_clause)
        content = cnf_file.read()
        cnf_file.seek(0, 0)
        cnf_file.write(header.rstrip('\r\n') + '\n' + content)
        cnf_file.close()
        return ret

    def visitBlock(self, ctx: CircuitSATParser.BlockContext):
        for fdx in ctx.functionDecl():
            if fdx not in self.store:
                self.visit(fdx)
        for sx in ctx.statement():
            self.visit(sx)
        if ctx.expression():
            value = self.visit(ctx.expression())
            ret = ReturnValue(value)
            raise ret
        return None

    def visitStatement(self, ctx: CircuitSATParser.StatementContext):
        return super().visitStatement(ctx)

    def visitAssignment(self, ctx: CircuitSATParser.AssignmentContext):
        new_val = self.visit(ctx.expression())
        if ctx.indexes():
            val = self.store[ctx.Identifier().getText()]
            for ec in ctx.indexes().expression():
                idx = self.visit(ec)
                val[idx] = new_val
        else:
            idx = ctx.Identifier().getText()
            self.store[idx] = new_val
        return None

    def visitIdentifierFunctionCall(self, ctx: CircuitSATParser.IdentifierFunctionCallContext):
        params = []
        if ctx.exprList():
            params = ctx.exprList().expression()
        idx = ctx.Identifier().getText() + str(len(params))
        return self.store[idx].invoke(self, params)

    def visitPrintlnFunctionCall(self, ctx: CircuitSATParser.PrintlnFunctionCallContext):
        if ctx.expression():
            print(self.visit(ctx.expression()))
        else:
            print()
        return None

    def visitPrintFunctionCall(self, ctx: CircuitSATParser.PrintFunctionCallContext):
        print(self.visit(ctx.expression()), end='')
        return None

    def visitAssertFunctionCall(self, ctx: CircuitSATParser.AssertFunctionCallContext):
        if not self.visit(ctx.expression()):
            raise Exception('Failed Assertion {} at line: {}'.format(ctx.expression().getText(), ctx.start.line))
        return None

    def visitSizeFunctionCall(self, ctx: CircuitSATParser.SizeFunctionCallContext):
        return len(self.visit(ctx.expression()))

    def visitIfStatement(self, ctx: CircuitSATParser.IfStatementContext):
        if self.visit(ctx.ifStat().expression()):
            return self.visit(ctx.ifStat().block())

        for i in range(len(ctx.elseIfStat())):
            if self.visit(ctx.elseIfStat(i).expression()):
                return self.visit(ctx.elseIfStat(i).block())

        if ctx.elseStat():
            return self.visit(ctx.elseStat().block())

    def visitIfStat(self, ctx: CircuitSATParser.IfStatContext):
        if self.visit(ctx.expression()):
            return self.visit(ctx.block())

    def visitElseIfStat(self, ctx: CircuitSATParser.ElseIfStatContext):
        if self.visit(ctx.expression()):
            return self.visit(ctx.block())

    def visitElseStat(self, ctx: CircuitSATParser.ElseStatContext):
        return self.visit(ctx.block())

    def visitFunctionDecl(self, ctx: CircuitSATParser.FunctionDeclContext):
        params = []
        if ctx.idList():
            params = ctx.idList().Identifier()
        idx = ctx.Identifier().getText() + str(len(params))
        block = ctx.block()
        self.store[idx] = Function(idx, params, block)
        return None

    def visitForStatement(self, ctx: CircuitSATParser.ForStatementContext):
        start = int(self.visit(ctx.expression(0)))
        stop = int(self.visit(ctx.expression(1)))
        for i in range(start, stop):
            self.store[ctx.Identifier().getText()] = i
            ret = self.visit(ctx.block())
            if ret:
                return ret
        return None

    def visitWhileStatement(self, ctx: CircuitSATParser.WhileStatementContext):
        while self.visit(ctx.expression()):
            val = self.visit(ctx.block())
            if val:
                return val
        return None

    def visitAssemblerStatement(self, ctx: CircuitSATParser.AssemblerStatementContext):
        return super().visitAssemblerStatement(ctx)

    def visitIdList(self, ctx: CircuitSATParser.IdListContext):
        return super().visitIdList(ctx)

    def visitExprList(self, ctx: CircuitSATParser.ExprListContext):
        return super().visitExprList(ctx)

    def visitBoolExpression(self, ctx: CircuitSATParser.BoolExpressionContext):
        return True if ctx.getText() == 'true' else False

    def visitNumberExpression(self, ctx: CircuitSATParser.NumberExpressionContext):
        return eval(ctx.getText())

    def visitIdentifierExpression(self, ctx: CircuitSATParser.IdentifierExpressionContext):
        idx = ctx.Identifier().getText()
        val = self.store[idx]
        if ctx.indexes():
            for ec in ctx.indexes().expression():
                idx = self.visit(ec)
                if isinstance(val, str):
                    val = list(val)[idx]
                else:
                    val = val[idx]
        return val

    def visitNotExpression(self, ctx: CircuitSATParser.NotExpressionContext):
        a = self.visit(ctx.expression()),
        return not a[0]

    def visitOrExpression(self, ctx: CircuitSATParser.OrExpressionContext):
        a, b = self.visit(ctx.expression(0)), self.visit(ctx.expression(1))
        return a | b

    def visitUnaryMinusExpression(self, ctx: CircuitSATParser.UnaryMinusExpressionContext):
        a = self.visit(ctx.expression()),
        return -1 * a[0]

    def visitPowerExpression(self, ctx: CircuitSATParser.PowerExpressionContext):
        a, b = self.visit(ctx.expression(0)), self.visit(ctx.expression(1))
        return a ** b

    def visitEqExpression(self, ctx: CircuitSATParser.EqExpressionContext):
        a, b = self.visit(ctx.expression(0)), self.visit(ctx.expression(1))
        if ctx.op.type == CircuitSATLexer.Equals:
            return a == b
        if ctx.op.type == CircuitSATLexer.NEquals:
            return a != b

    def visitAndExpression(self, ctx: CircuitSATParser.AndExpressionContext):
        a, b = self.visit(ctx.expression(0)), self.visit(ctx.expression(1))
        return a & b

    def visitInExpression(self, ctx: CircuitSATParser.InExpressionContext):
        a, b = self.visit(ctx.expression(0)), self.visit(ctx.expression(1))
        return a in b

    def visitStringExpression(self, ctx: CircuitSATParser.StringExpressionContext):
        return ctx.getText().replace('\'', '').replace('\"', '')

    def visitExpressionExpression(self, ctx: CircuitSATParser.ExpressionExpressionContext):
        val = self.visit(ctx.expression())
        if ctx.indexes():
            for ec in ctx.indexes():
                idx = self.visit(ec)
                val = list(val)[idx]
        return val

    def visitAddExpression(self, ctx: CircuitSATParser.AddExpressionContext):
        a, b = self.visit(ctx.expression(0)), self.visit(ctx.expression(1))
        if ctx.op.type == CircuitSATLexer.Add:
            if isinstance(a, list):
                a.append(b)
                return a
            return a + b
        if ctx.op.type == CircuitSATLexer.Subtract:
            if isinstance(a, list):
                if b in a:
                    a.remove(b)
                return a
            return a - b

    def visitCompExpression(self, ctx: CircuitSATParser.CompExpressionContext):
        a, b = self.visit(ctx.expression(0)), self.visit(ctx.expression(1))
        if ctx.op.type == CircuitSATLexer.LT:
            return a < b
        if ctx.op.type == CircuitSATLexer.LTEquals:
            return a <= b
        if ctx.op.type == CircuitSATLexer.GT:
            return a > b
        if ctx.op.type == CircuitSATLexer.GTEquals:
            return a >= b

    def visitNullExpression(self, ctx: CircuitSATParser.NullExpressionContext):
        return None

    def visitFunctionCallExpression(self, ctx: CircuitSATParser.FunctionCallExpressionContext):
        val = self.visit(ctx.functionCall())
        if ctx.indexes():
            for ec in ctx.indexes().expression():
                return val[self.visit(ec)]
        return val

    def visitMultExpression(self, ctx: CircuitSATParser.MultExpressionContext):
        a, b = self.visit(ctx.expression(0)), self.visit(ctx.expression(1))
        if ctx.op.type == CircuitSATLexer.Multiply:
            if isinstance(a, str):
                return a.replace('\'', '').replace('\"', '') * b
            elif isinstance(b, str):
                return a * b.replace('\'', '').replace('\"', '')
            return a * b
        if ctx.op.type == CircuitSATLexer.Divide:
            return a / b
        if ctx.op.type == CircuitSATLexer.Modulus:
            return a % b
        if ctx.op.type == CircuitSATLexer.IDivide:
            return a // b

    def visitListExpression(self, ctx: CircuitSATParser.ListExpressionContext):
        val = self.visit(ctx.list_())
        if ctx.indexes():
            for ec in ctx.indexes():
                idx = self.visit(ec)
                if isinstance(val, str):
                    return list(val)[idx]
                else:
                    return val[idx]
        return val

    def visitTernaryExpression(self, ctx: CircuitSATParser.TernaryExpressionContext):
        return super().visitTernaryExpression(ctx)

    def visitInputExpression(self, ctx: CircuitSATParser.InputExpressionContext):
        raise Exception('TODO: Implement')
        return super().visitInputExpression(ctx)

    def visitList(self, ctx: CircuitSATParser.ListContext):
        ls = []
        if ctx.exprList():
            for ex in ctx.exprList().expression():
                ls.append(self.visit(ex))
        return ls

    def visitIndexes(self, ctx: CircuitSATParser.IndexesContext):
        return super().visitIndexes(ctx)

    def visitOpcode(self, ctx: CircuitSATParser.OpcodeContext):
        opcode = ctx.getText()
        if opcode.startswith('new'):
            x = ctx.Identifier(0).getText()
            if ctx.expression():
                self.apply_new(x + str(self.visit(ctx.expression(0))))
            else:
                self.apply_new(x)
        elif opcode.startswith('copy'):
            x, y = ctx.Identifier(0).getText(), ctx.Identifier(1).getText()

            a = ctx.expression(0)
            b = ctx.expression(1)

            if a is None:
                a = x
            else:
                a = x + str(self.visit(ctx.expression(0)))

            if b is None:
                b = y
            else:
                b = y + str(self.visit(ctx.expression(1)))

            self.apply_copy(a, b)
        elif opcode.startswith('not'):
            x, y = ctx.Identifier(0).getText(), ctx.Identifier(1).getText()
            a = ctx.expression(0)
            b = ctx.expression(1)

            if a is None:
                a = x
            else:
                a = x + str(self.visit(ctx.expression(0)))

            if b is None:
                b = y
            else:
                b = y + str(self.visit(ctx.expression(1)))

            self.apply_not(a, b)
        elif opcode.startswith('and'):
            x, y, z = ctx.Identifier(0).getText(), ctx.Identifier(1).getText(), ctx.Identifier(2).getText()

            a = ctx.expression(0)
            b = ctx.expression(1)
            c = ctx.expression(2)

            if a is None:
                a = x
            else:
                a = x + str(self.visit(ctx.expression(0)))

            if b is None:
                b = y
            else:
                b = y + str(self.visit(ctx.expression(1)))

            if c is None:
                c = z
            else:
                c = z + str(self.visit(ctx.expression(2)))

            self.apply_and(a, b, c)
        elif opcode.startswith('or'):
            x, y, z = ctx.Identifier(0).getText(), ctx.Identifier(1).getText(), ctx.Identifier(2).getText()

            a = ctx.expression(0)
            b = ctx.expression(1)
            c = ctx.expression(2)

            if a is None:
                a = x
            else:
                a = x + str(self.visit(ctx.expression(0)))

            if b is None:
                b = y
            else:
                b = y + str(self.visit(ctx.expression(1)))

            if c is None:
                c = z
            else:
                c = z + str(self.visit(ctx.expression(2)))

            self.apply_or(a, b, c)
        elif opcode.startswith('xor'):
            x, y, z = ctx.Identifier(0).getText(), ctx.Identifier(1).getText(), ctx.Identifier(2).getText()

            a = ctx.expression(0)
            b = ctx.expression(1)
            c = ctx.expression(2)

            if a is None:
                a = x
            else:
                a = x + str(self.visit(ctx.expression(0)))

            if b is None:
                b = y
            else:
                b = y + str(self.visit(ctx.expression(1)))

            if c is None:
                c = z
            else:
                c = z + str(self.visit(ctx.expression(2)))

            self.apply_xor(a, b, c)
        elif opcode.startswith('equ'):
            x, y, z = ctx.Identifier(0).getText(), ctx.Identifier(1).getText(), ctx.Identifier(2).getText()

            a = ctx.expression(0)
            b = ctx.expression(1)
            c = ctx.expression(2)

            if a is None:
                a = x
            else:
                a = x + str(self.visit(ctx.expression(0)))

            if b is None:
                b = y
            else:
                b = y + str(self.visit(ctx.expression(1)))

            if c is None:
                c = z
            else:
                c = z + str(self.visit(ctx.expression(2)))

            self.apply_equ(a, b, c)
        elif opcode.startswith('nor'):
            x, y, z = ctx.Identifier(0).getText(), ctx.Identifier(1).getText(), ctx.Identifier(2).getText()

            a = ctx.expression(0)
            b = ctx.expression(1)
            c = ctx.expression(2)

            if a is None:
                a = x
            else:
                a = x + str(self.visit(ctx.expression(0)))

            if b is None:
                b = y
            else:
                b = y + str(self.visit(ctx.expression(1)))

            if c is None:
                c = z
            else:
                c = z + str(self.visit(ctx.expression(2)))

            self.apply_nor(a, b, c)
        elif opcode.startswith('nand'):
            x, y, z = ctx.Identifier(0).getText(), ctx.Identifier(1).getText(), ctx.Identifier(2).getText()

            a = ctx.expression(0)
            b = ctx.expression(1)
            c = ctx.expression(2)

            if a is None:
                a = x
            else:
                a = x + str(self.visit(ctx.expression(0)))

            if b is None:
                b = y
            else:
                b = y + str(self.visit(ctx.expression(1)))

            if c is None:
                c = z
            else:
                c = z + str(self.visit(ctx.expression(2)))

            self.apply_nand(a, b, c)
        elif opcode.startswith('put'):
            x, y = ctx.Identifier(0).getText(), ctx.Identifier(1).getText()

            a = ctx.expression(0)
            b = ctx.expression(1)

            if a is None:
                a = x
            else:
                a = x + str(self.visit(ctx.expression(0)))

            if b is None:
                b = y
            else:
                b = y + str(self.visit(ctx.expression(1)))

            self.apply_put(a, b)
        return super().visitOpcode(ctx)

    def add_clauses(self, clauses):
        for cls in clauses:
            self.current_clause += 1
            print(' '.join(list(map(str, cls))) + ' 0', file=self.cnf_file)

    def add_comment(self, comment):
        self.current_clause += 1
        print('c {}'.format(comment), file=self.cnf_file)

    def apply_new(self, x):
        self.current_literal += 1
        self.store[x] = self.current_literal

    def apply_put(self, x, y):
        if x not in self.store:
            if '[' in x and x[:x.index('[')] in self.store:
                xl = self.store[x[:x.index('[')]]
                x_ = xl[eval(x[x.index('[') + 1:-1])]
                self.store[x] = x_
        if y not in self.store:
            if '[' in y and y[:y.index('[')] in self.store:
                yl = self.store[y[:y.index('[')]]
                y_ = yl[eval(y[y.index('[') + 1:-1])]
                self.store[y] = y_
        x, y = self.store[x], self.store[y]
        if y:
            self.add_clauses([[x]])
        else:
            self.add_clauses([[-1 * x]])

    def apply_not(self, x, y):
        if x not in self.store:
            if '[' in x and x[:x.index('[')] in self.store:
                xl = self.store[x[:x.index('[')]]
                x_ = xl[eval(x[x.index('[') + 1:-1])]
                self.store[x] = x_
        if y not in self.store:
            if '[' in y and y[:y.index('[')] in self.store:
                yl = self.store[y[:y.index('[')]]
                y_ = yl[eval(y[y.index('[') + 1:-1])]
                self.store[y] = y_
        x, y = self.store[x], self.store[y]
        self.add_clauses([[x, y],
                          [-x, -y]])

    def apply_copy(self, x, y):
        if x not in self.store:
            if '[' in x and x[:x.index('[')] in self.store:
                xl = self.store[x[:x.index('[')]]
                x_ = xl[eval(x[x.index('[') + 1:-1])]
                self.store[x] = x_
        if y not in self.store:
            if '[' in y and y[:y.index('[')] in self.store:
                yl = self.store[y[:y.index('[')]]
                y_ = yl[eval(y[y.index('[') + 1:-1])]
                self.store[y] = y_
        x, y = self.store[x], self.store[y]
        self.add_clauses([[x, -y],
                          [-x, y]])

    def apply_and(self, x, y, z):
        if x not in self.store:
            if '[' in x and x[:x.index('[')] in self.store:
                xl = self.store[x[:x.index('[')]]
                x_ = xl[eval(x[x.index('[') + 1:-1])]
                self.store[x] = x_
        if y not in self.store:
            if '[' in y and y[:y.index('[')] in self.store:
                yl = self.store[y[:y.index('[')]]
                y_ = yl[eval(y[y.index('[') + 1:-1])]
                self.store[y] = y_
        if z not in self.store:
            if '[' in z and z[:z.index('[')] in self.store:
                zl = self.store[z[:z.index('[')]]
                z_ = zl[eval(z[z.index('[') + 1:-1])]
                self.store[z] = z_
        x, y, z = self.store[x], self.store[y], self.store[z]
        self.add_clauses([[x, y, -z],
                          [x, -y, -z],
                          [-x, y, -z],
                          [-x, -y, z]])

    def apply_nand(self, x, y, z):
        if x not in self.store:
            if '[' in x and x[:x.index('[')] in self.store:
                xl = self.store[x[:x.index('[')]]
                x_ = xl[eval(x[x.index('[') + 1:-1])]
                self.store[x] = x_
        if y not in self.store:
            if '[' in y and y[:y.index('[')] in self.store:
                yl = self.store[y[:y.index('[')]]
                y_ = yl[eval(y[y.index('[') + 1:-1])]
                self.store[y] = y_
        if z not in self.store:
            if '[' in z and z[:z.index('[')] in self.store:
                zl = self.store[z[:z.index('[')]]
                z_ = zl[eval(z[z.index('[') + 1:-1])]
                self.store[z] = z_
        x, y, z = self.store[x], self.store[y], self.store[z]
        self.add_clauses([[x, y, z],
                          [x, -y, z],
                          [-x, y, z],
                          [-x, -y, -z]])

    def apply_or(self, x, y, z):
        if x not in self.store:
            if '[' in x and x[:x.index('[')] in self.store:
                xl = self.store[x[:x.index('[')]]
                x_ = xl[eval(x[x.index('[') + 1:-1])]
                self.store[x] = x_
        if y not in self.store:
            if '[' in y and y[:y.index('[')] in self.store:
                yl = self.store[y[:y.index('[')]]
                y_ = yl[eval(y[y.index('[') + 1:-1])]
                self.store[y] = y_
        if z not in self.store:
            if '[' in z and z[:z.index('[')] in self.store:
                zl = self.store[z[:z.index('[')]]
                z_ = zl[eval(z[z.index('[') + 1:-1])]
                self.store[z] = z_
        x, y, z = self.store[x], self.store[y], self.store[z]
        self.add_clauses([[x, y, -z],
                          [x, -y, z],
                          [-x, y, z],
                          [-x, -y, z]])

    def apply_nor(self, x, y, z):
        if x not in self.store:
            if '[' in x and x[:x.index('[')] in self.store:
                xl = self.store[x[:x.index('[')]]
                x_ = xl[eval(x[x.index('[') + 1:-1])]
                self.store[x] = x_
        if y not in self.store:
            if '[' in y and y[:y.index('[')] in self.store:
                yl = self.store[y[:y.index('[')]]
                y_ = yl[eval(y[y.index('[') + 1:-1])]
                self.store[y] = y_
        if z not in self.store:
            if '[' in z and z[:z.index('[')] in self.store:
                zl = self.store[z[:z.index('[')]]
                z_ = zl[eval(z[z.index('[') + 1:-1])]
                self.store[z] = z_
        x, y, z = self.store[x], self.store[y], self.store[z]
        self.add_clauses([[x, y, z],
                          [x, -y, -z],
                          [-x, y, -z],
                          [-x, -y, -z]])

    def apply_xor(self, x, y, z):
        if x not in self.store:
            if '[' in x and x[:x.index('[')] in self.store:
                xl = self.store[x[:x.index('[')]]
                x_ = xl[eval(x[x.index('[') + 1:-1])]
                self.store[x] = x_
        if y not in self.store:
            if '[' in y and y[:y.index('[')] in self.store:
                yl = self.store[y[:y.index('[')]]
                y_ = yl[eval(y[y.index('[') + 1:-1])]
                self.store[y] = y_
        if z not in self.store:
            if '[' in z and z[:z.index('[')] in self.store:
                zl = self.store[z[:z.index('[')]]
                z_ = zl[eval(z[z.index('[') + 1:-1])]
                self.store[z] = z_
        x, y, z = self.store[x], self.store[y], self.store[z]
        self.add_clauses([[x, y, -z],
                          [x, -y, z],
                          [-x, y, z],
                          [-x, -y, -z]])

    def apply_equ(self, x, y, z):
        if x not in self.store:
            if '[' in x and x[:x.index('[')] in self.store:
                xl = self.store[x[:x.index('[')]]
                x_ = xl[eval(x[x.index('[') + 1:-1])]
                self.store[x] = x_
        if y not in self.store:
            if '[' in y and y[:y.index('[')] in self.store:
                yl = self.store[y[:y.index('[')]]
                y_ = yl[eval(y[y.index('[') + 1:-1])]
                self.store[y] = y_
        if z not in self.store:
            if '[' in z and z[:z.index('[')] in self.store:
                zl = self.store[z[:z.index('[')]]
                z_ = zl[eval(z[z.index('[') + 1:-1])]
                self.store[z] = z_
        x, y, z = self.store[x], self.store[y], self.store[z]
        self.add_clauses([[x, y, z],
                          [x, -y, -z],
                          [-x, y, -z],
                          [-x, -y, z]])

    def apply_cur(self, name):
        if name in self.store:
            self.current_literal = self.store[name]
        else:
            self.current_literal = int(name) - 1



# Generated from C:/Users/oscar/Downloads/Development/TL\CircuitSAT.g4 by ANTLR 4.10.1
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .CircuitSATParser import CircuitSATParser
else:
    from CircuitSATParser import CircuitSATParser

# This class defines a complete generic visitor for a parse tree produced by CircuitSATParser.

class CircuitSATVisitor(ParseTreeVisitor):

    # Visit a parse tree produced by CircuitSATParser#circuit.
    def visitCircuit(self, ctx:CircuitSATParser.CircuitContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#block.
    def visitBlock(self, ctx:CircuitSATParser.BlockContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#statement.
    def visitStatement(self, ctx:CircuitSATParser.StatementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#assignment.
    def visitAssignment(self, ctx:CircuitSATParser.AssignmentContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#identifierFunctionCall.
    def visitIdentifierFunctionCall(self, ctx:CircuitSATParser.IdentifierFunctionCallContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#printlnFunctionCall.
    def visitPrintlnFunctionCall(self, ctx:CircuitSATParser.PrintlnFunctionCallContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#printFunctionCall.
    def visitPrintFunctionCall(self, ctx:CircuitSATParser.PrintFunctionCallContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#assertFunctionCall.
    def visitAssertFunctionCall(self, ctx:CircuitSATParser.AssertFunctionCallContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#sizeFunctionCall.
    def visitSizeFunctionCall(self, ctx:CircuitSATParser.SizeFunctionCallContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#ifStatement.
    def visitIfStatement(self, ctx:CircuitSATParser.IfStatementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#ifStat.
    def visitIfStat(self, ctx:CircuitSATParser.IfStatContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#elseIfStat.
    def visitElseIfStat(self, ctx:CircuitSATParser.ElseIfStatContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#elseStat.
    def visitElseStat(self, ctx:CircuitSATParser.ElseStatContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#functionDecl.
    def visitFunctionDecl(self, ctx:CircuitSATParser.FunctionDeclContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#forStatement.
    def visitForStatement(self, ctx:CircuitSATParser.ForStatementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#whileStatement.
    def visitWhileStatement(self, ctx:CircuitSATParser.WhileStatementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#assemblerStatement.
    def visitAssemblerStatement(self, ctx:CircuitSATParser.AssemblerStatementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#idList.
    def visitIdList(self, ctx:CircuitSATParser.IdListContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#exprList.
    def visitExprList(self, ctx:CircuitSATParser.ExprListContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#boolExpression.
    def visitBoolExpression(self, ctx:CircuitSATParser.BoolExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#numberExpression.
    def visitNumberExpression(self, ctx:CircuitSATParser.NumberExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#identifierExpression.
    def visitIdentifierExpression(self, ctx:CircuitSATParser.IdentifierExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#notExpression.
    def visitNotExpression(self, ctx:CircuitSATParser.NotExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#orExpression.
    def visitOrExpression(self, ctx:CircuitSATParser.OrExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#unaryMinusExpression.
    def visitUnaryMinusExpression(self, ctx:CircuitSATParser.UnaryMinusExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#powerExpression.
    def visitPowerExpression(self, ctx:CircuitSATParser.PowerExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#eqExpression.
    def visitEqExpression(self, ctx:CircuitSATParser.EqExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#andExpression.
    def visitAndExpression(self, ctx:CircuitSATParser.AndExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#inExpression.
    def visitInExpression(self, ctx:CircuitSATParser.InExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#stringExpression.
    def visitStringExpression(self, ctx:CircuitSATParser.StringExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#expressionExpression.
    def visitExpressionExpression(self, ctx:CircuitSATParser.ExpressionExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#addExpression.
    def visitAddExpression(self, ctx:CircuitSATParser.AddExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#compExpression.
    def visitCompExpression(self, ctx:CircuitSATParser.CompExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#nullExpression.
    def visitNullExpression(self, ctx:CircuitSATParser.NullExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#functionCallExpression.
    def visitFunctionCallExpression(self, ctx:CircuitSATParser.FunctionCallExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#multExpression.
    def visitMultExpression(self, ctx:CircuitSATParser.MultExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#listExpression.
    def visitListExpression(self, ctx:CircuitSATParser.ListExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#ternaryExpression.
    def visitTernaryExpression(self, ctx:CircuitSATParser.TernaryExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#inputExpression.
    def visitInputExpression(self, ctx:CircuitSATParser.InputExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#list.
    def visitList(self, ctx:CircuitSATParser.ListContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#indexes.
    def visitIndexes(self, ctx:CircuitSATParser.IndexesContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by CircuitSATParser#opcode.
    def visitOpcode(self, ctx:CircuitSATParser.OpcodeContext):
        return self.visitChildren(ctx)



del CircuitSATParser